{$O+A+W-U-I+B-X+T-P+H+J+R-Q-D-L-}

//******************************************************
//******************************************************
// Project: Delphi Initialization Database
// Author: JBD
// E-mail: binel@alf.tel.hr
// Comment: ...
//******************************************************
//******************************************************

unit INIDsgn;

interface

uses
	SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
	Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, INISec,INIEnt,Mask, 
  INIMis,ComCtrls,DsgnIntf,INIFile{$ifndef VER90},LibIntf{$endif}{$ifdef VER90},LibMain{$endif};

type
  TSubProperties=class(TClassProperty)
  protected
  	function GetAttributes:TPropertyAttributes; override;
  end;

  TNamesProperty=class(TSubProperties)
  end;

  TDirsProperty=class(TSubProperties)
  end;

  TGlyphsProperty=class(TSubProperties)
  end;

  TGlyphDescriptorProperty=class(TSubProperties)
  end;

  TFormNameProperty=class(TComponentProperty)
  protected
  	procedure GetValues(Proc:TGetStrProc); override;

    procedure SetValue(const Name:string); override;
  end;

  TINIDatabaseEditor=class(TDefaultEditor)
  public
    procedure ExecuteVerb(Index:Integer); override;
    function GetVerb(Index:Integer):string; override;
    function GetVerbCount:Integer; override;

		procedure EditProperty(PropertyEditor:TPropertyEditor;var Continue,FreeEditor:Boolean); override;

		procedure GoExplore;
		procedure GoEdit;
		procedure GoTest;
		procedure GoAbout;
  end;

	TGroupNameProperty=class(TStringProperty)
  protected
  	function GetAttributes:TPropertyAttributes; override;

  	procedure GetValues(Proc:TGetStrProc); override;
  end;

procedure ExploreDatabase(Designer:TFormDesigner;Database:TINIDatabase;ReCreate:Boolean);
procedure ShowInspector;

implementation

uses INITest,INIDsgnF,INIAbout,INIEdit;

function TSubProperties.GetAttributes:TPropertyAttributes;
begin
	Result:=[paSubProperties];
end;

procedure TFormNameProperty.GetValues(Proc:TGetStrProc);
var i:Integer;
begin
	for i:=0 to Screen.FormCount-1 do begin
  	Proc(Screen.Forms[i].Name);
  end;

	for i:=0 to Screen.DataModuleCount-1 do begin
  	Proc(Screen.DataModules[i].Name);
  end;
end;

procedure TFormNameProperty.SetValue(const Name:string);
var i:Integer;
begin
	for i:=0 to Screen.FormCount-1 do begin
  	if Screen.Forms[i].Name=Name then begin
    	SetOrdValue(LongInt(Screen.Forms[i]));

    	Exit;
    end;
  end;

 	SetOrdValue(0);
end;

function TINIDatabaseEditor.GetVerbCount:Integer;
begin
  Result:=7;
end;

function TINIDatabaseEditor.GetVerb(Index:Integer):string;
begin
  if Index=0 then
  	Result:='&About ...'
  else if Index=1 then
  	Result:='-'
  else if Index=2 then
    Result:='E&xplore ...'
  else if Index=3 then
  	Result:='&Test ...'
  else if Index=4 then
  	Result:='-'
  else if Index=5 then
  	Result:='&Edit ...'
  else if Index=6 then
  	Result:='&Inspect'
  else
  	Result:='';
end;

procedure TINIDatabaseEditor.ExecuteVerb(Index:Integer);
begin
  if Index=0 then
  	GoAbout
	else if Index=2 then
	  GoExplore
  else if Index=3 then
  	GoTest
  else if Index=5 then
  	GoEdit
  else if Index=6 then
  	ShowInspector
end;

procedure TINIDatabaseEditor.EditProperty(PropertyEditor:TPropertyEditor;var Continue,FreeEditor:Boolean);
begin
	if CompareText(PropertyEditor.GetName,'DIRS')=0 then GoExplore;
end;

procedure TINIDatabaseEditor.GoExplore;
begin
	if Component is TINIDatabase then begin
  	ExploreDatabase(Designer,Component as TINIDatabase,False);

    Designer.Modified;
  end;
end;

procedure TINIDatabaseEditor.GoEdit;
var frmEditINIDB:TfrmEditINIDB;
begin
	if Component is TINIDatabase then begin
    frmEditINIDB:=TfrmEditINIDB.Create(nil);

    try
      frmEditINIDB.Go(Designer,Component as TINIDatabase);

      finally
        frmEditINIDB.Free;
    end;
  end;
end;

procedure TINIDatabaseEditor.GoAbout;
var frmAbout:TfrmAbout;
begin
	frmAbout:=TfrmAbout.Create(nil);

  try
  	frmAbout.ShowModal;
    
  	finally
    	frmAbout.Free;
  end;
end;

procedure TINIDatabaseEditor.GoTest;
var frmTestINIDatabase:TfrmTestINIDatabase;
begin
	if Component<>nil then begin
    frmTestINIDatabase:=TfrmTestINIDatabase.Create(Application);

    try
      frmTestINIDatabase.GoTest(Component as TINIDatabase,nil,nil,'',dmEdit,True);

      Designer.Modified;

      finally
        frmTestINIDatabase.Free;
    end;
  end;
end;

function TGroupNameProperty.GetAttributes:TPropertyAttributes;
begin
	Result:=[paMultiSelect,paValueList];
end;

procedure TGroupNameProperty.GetValues(Proc:TGetStrProc);
var INIS:TINICustomSection;
		i:Integer;
begin
	if GetComponent(0) is TINIEntry then with GetComponent(0) as TINIEntry do begin
		if Table is TINICustomSection then begin
    	INIS:=Table as TINICustomSection;

	    for i:=0 to INIS.GroupCount-1 do Proc(INIS.Group[i]);
    end;
  end;
end;

procedure ShowInspector;
{$ifndef VER90}
var OIHandle:HWND;

	function GetObjectInspector(const TaskName,ClassName:string):HWND;
  var cTaskName,cClassName:string;
      lBuffer:array[0..127] of char;
      hw:HWND;
  begin
    hw:=GetWindow(GetDesktopWindow,GW_CHILD);

    while IsWindow(hw) do begin
      WinProcs.GetWindowText(hw,lBuffer,127);
      cTaskName:=StrPas(lBuffer);

      WinProcs.GetClassName(hw,lBuffer,127);
      cClassName:=StrPas(lBuffer);

      if StrNICmp(TaskName,cTaskName,Length(TaskName)) and StrNICmp(ClassName,cClassName,Length(ClassName)) then begin
        Result:=hw;

        Exit;
      end;

      hw:=GetWindow(hw,GW_HWNDNEXT);
    end;

    Result:=0;
  end;
{$endif}

begin
{$ifdef VER90}
	AppBuilder.ModalEdit(#0,Application);
{$endif}
{$ifndef VER90}
	OIHandle:=GetObjectInspector('Object Inspector','TPropertyInspector');

  if OIHandle<>0 then begin
  	WinProcs.ShowWindow(OIHandle,SW_RESTORE);
    InvalidateRect(OIHandle,nil,False);
  	WinProcs.BringWindowToTop(OIHandle);
  end;
{$endif}
end;

procedure ExploreDatabase(Designer:TFormDesigner;Database:TINIDatabase;ReCreate:Boolean);
begin
  if frmINIDatabase=nil then frmINIDatabase:=TfrmINIDatabase.Create(nil);

  if (frmINIDatabase.Visible) and (frmINIDatabase.INIDatabase=Database) and (not ReCreate) then begin
    frmINIDatabase.WindowState:=wsNormal;
    frmINIDatabase.BringToFront;
  end else begin
    frmINIDatabase.Go(Designer,Database,Database.Master);
  end;
end;

end.
