{$O+A+W-U-I+B-X+T-P+H+J+R-Q-D-L-}

//******************************************************
//******************************************************
// Project: Delphi Initialization Database
// Author: JBD
// E-mail: binel@alf.tel.hr
// Comment: Additional "INI" entries (font name, printer name, ...)
//******************************************************
//******************************************************

unit INIEntEx;

interface

uses INISec,INIEnt,Classes,Controls,Chkgroup,StdCtrls,ComCtrls,INIMis,Forms,
		 SysUtils,Printers;

type
  TINIEntrySaveListForCheckGroup=class(TINIEntrySaveList)
  public
    procedure Syncronize(SD:TSyncronizeDirection); override;

    function IsConnectionToControlSupported(WC:TWinControl):Boolean; override;
  end;

  TINIEntrySaveListForListView=class(TINIEntrySaveList)
  public
    procedure Syncronize(SD:TSyncronizeDirection); override;

    function IsConnectionToControlSupported(WC:TWinControl):Boolean; override;
  end;

  TINIEntryForRadioButton=class(TINIEntryBoolean)
  public
    procedure Syncronize(SD:TSyncronizeDirection); override;

    function IsConnectionToControlSupported(WC:TWinControl):Boolean; override;
  end;

	TINIEntryDate=class(TINIEntryString)
  protected
		function GetWinControl(AOwner:TComponent;WParent:TControl;cInfo:TCreateInfo):TControl; override;

		function GetDefaultWidth:Integer; override;

    function GetAsDate:TDateTime; virtual;
    procedure SetAsDate(Date:TDateTime); virtual;

  public
  	constructor Create(AOwner:TComponent); override;
    
		procedure RefreshControl; override;

    property AsDate:TDateTime read GetAsDate write SetAsDate;
  end;

  TINIEntryFontName=class(TINIEntryCustomOneOfStrings)
  protected
  	procedure Init; override;

    function StoreItems:Boolean; override;

  published
  	property EditControl;
    property DisplayHeight;
  end;

  TINIEntryPrinterName=class(TINIEntryCustomOneOfStrings)
  protected
  	procedure Init; override;

    function StoreItems:Boolean; override;

  published
  	property EditControl;
    property DisplayHeight;
  end;

procedure Register;

implementation

uses INICtrEx;

function TINIEntrySaveListForCheckGroup.IsConnectionToControlSupported(WC:TWinControl):Boolean;
begin
	Result:=inherited IsConnectionToControlSupported(WC) or (WC is TCheckGroup);
end;

procedure TINIEntrySaveListForCheckGroup.Syncronize(SD:TSyncronizeDirection);
var i,cc:Integer;
    CG:TCheckGroup;
begin
  if ConnectedControl is TCheckGroup then begin
    CG:=ConnectedControl as TCheckGroup;

    if SD=sdToControl then begin
      cc:=CG.Items.Count-1;

      for i:=0 to cc do begin
        if List.IndexOf(CG.Items[i])<>-1 then
          CG.CheckItem(i)
        else
          CG.UnCheckItem(i)
      end;
    end else begin
      List.Clear;

      cc:=CG.Items.Count-1;

      for i:=0 to cc do begin
        if CG.ItemChecked(i) then List.Add(CG.Items[i]);
      end;
    end;
  end else inherited;
end;

{$H-}
procedure TINIEntrySaveListForListView.Syncronize(SD:TSyncronizeDirection);
var i,cc:Integer;
    LV:TListView;
    LI:TListItem;
begin
  if ConnectedControl is TListView then begin
    LV:=ConnectedControl as TListView;

    if SD=sdToControl then begin
      LV.Items.Clear;

      cc:=List.Count-1;
      
      for i:=0 to cc do begin
        LI:=LV.Items.Add;

        LI.Caption:=Copy(List[i],1,8);
        LI.SubItems.Add(Copy(List[i],9,Length(List[i])-8));
      end;
    end else begin
      List.Clear;

      cc:=LV.Items.Count-1;

      for i:=0 to cc do begin
        List.Add(PadRight(' ',8,LV.Items[i].Caption)+LV.Items[i].SubItems[0]);
      end;
    end;
  end else inherited;
end;

function TINIEntrySaveListForListView.IsConnectionToControlSupported(WC:TWinControl):Boolean;
begin
	Result:=inherited IsConnectionToControlSupported(WC) or (WC is TListView);
end;

{$H+}

procedure TINIEntryForRadioButton.Syncronize(SD:TSyncronizeDirection);
begin
  if ConnectedControl is TRadioButton then with ConnectedControl as TRadioButton do begin
    if SD=sdToControl then
      Checked:=AsBoolean
    else
      AsBoolean:=Checked;
  end else inherited;
end;

function TINIEntryForRadioButton.IsConnectionToControlSupported(WC:TWinControl):Boolean;
begin
	Result:=inherited IsConnectionToControlSupported(WC) or (WC is TRadioButton);
end;

constructor TINIEntryDate.Create(AOwner:TComponent);
begin
	inherited Create(AOwner);

  FDisplayWidth:=10;
end;

function TINIEntryDate.GetAsDate:TDateTime;
begin
	try
		Result:=StrToDate(AsString);

    except
    	on EConvertError do Result:=0;
  end;
end;

procedure TINIEntryDate.SetAsDate(Date:TDateTime);
begin
	AsString:=DateToStr(Date);
end;

function TINIEntryDate.GetDefaultWidth:Integer;
begin
  Result:=10;
end;

function TINIEntryDate.GetWinControl(AOwner:TComponent;WParent:TControl;cInfo:TCreateInfo):TControl;
begin
	with cInfo do begin
		Result:=INISec.CreateTitledControl(AOwner,TCfgDateEdit,WParent,cInfo) as TCfgDateEdit;
	end;

	(Result as TCfgDateEdit).INIEntry:=Self;
end;

procedure TINIEntryDate.RefreshControl;
begin
	if Control<>nil then (Control as TCfgDateEdit).INIEntry:=Self;
end;

procedure TINIEntryFontName.Init;
begin
	AssignItems(Screen.Fonts,Screen.Fonts);
end;

function TINIEntryFontName.StoreItems:Boolean;
begin
	Result:=False;
end;

procedure TINIEntryPrinterName.Init;
begin
	AssignItems(Printer.Printers,Printer.Printers);
end;

function TINIEntryPrinterName.StoreItems:Boolean;
begin
	Result:=False;
end;

procedure Register;
begin
	RegisterINIEntry('Date',eiiDate,TINIEntryDate);
	RegisterINIEntry('Font name',eiiFont,TINIEntryFontName);
	RegisterINIEntry('Printer name',eiiPrinter,TINIEntryPrinterName);
end;

end.
