{$O+A+W-U-I+B-X+T-P+H+J+R-Q-D-L-}

//******************************************************
//******************************************************
// Project: Delphi Initialization Database
// Author: JBD
// E-mail: binel@alf.tel.hr
// Comment: Component for saving form size and location. Uses TINIMultiSectionFormSaver.
//******************************************************
//******************************************************

unit INISaver;

interface

uses
	Classes, Controls, Messages, WinTypes, Forms, Graphics, WinProcs, StdCtrls,
	SysUtils, INIMis,INISecEx;

type
  TOnGetSectionName=procedure(Sender:TObject;var SectionName:string) of object;

	TINIFormSaver=class(TComponent)
	private
    FOnGetSectionName:TOnGetSectionName;
    FSection:TINIMultiSectionFormSaver;
    FCloseEvent:TCloseQueryEvent;

    procedure FormCloseQuery(Sender:TObject;var CanClose:Boolean);

	public
		constructor Create(Owner:TComponent); override;

    procedure SetFormPos(Form:TForm);

	protected
		FParent:TForm;

		procedure Loaded; override;

		function GetSectionName(const Name:string):string; virtual;

    procedure Notification(AComponent:TComponent;Operation:TOperation); override;

	published
  	property Section:TINIMultiSectionFormSaver read FSection write FSection;

    property OnGetSectionName:TOnGetSectionName read FOnGetSectionName write FOnGetSectionName;
	end;

procedure Register;

implementation

constructor TINIFormSaver.Create(Owner:TComponent);
begin
	inherited Create(Owner);

  FOnGetSectionName:=nil;

	FSection:=nil;
  FParent:=nil;
  FCloseEvent:=nil;
end;

procedure TINIFormSaver.FormCloseQuery(Sender:TObject;var CanClose:Boolean);
begin
	try
    if not (csDesigning in ComponentState) then begin
      if (FParent is TForm) and (FSection<>nil) then begin
        FSection.AddKey(GetSectionName(FParent.ClassName));

        FSection.FieldByName('X').AsInteger:=FParent.Left;
        FSection.FieldByName('Y').AsInteger:=FParent.Top;

        if not (FParent.BorderStyle in [bsSingle,bsDialog]) then begin
          FSection.FieldByName('Width').AsInteger:=FParent.Width;
          FSection.FieldByName('Height').AsInteger:=FParent.Height;
        end;

        FSection.FieldByName('State').AsInteger:=Integer(FParent.WindowState);

        FSection.WriteSection;
      end;
    end;

    finally
			if Assigned(FCloseEvent) then FCloseEvent(Sender,CanClose);
	end;
end;

procedure TINIFormSaver.Loaded;
var W,H,X,Y:Integer;
		SectionName:string;
begin
	inherited Loaded;

	FParent:=nil;

	if not (csDesigning in ComponentState) then begin
		if (Owner is TForm) and (FSection<>nil) then begin
			FParent:=Owner as TForm;
      FCloseEvent:=FParent.OnCloseQuery;
      FParent.OnCloseQuery:=FormCloseQuery;

      SectionName:=GetSectionName(FParent.ClassName);

      FSection.Reset;
      FSection.ReadSection;

      if FSection.FindKey(SectionName) then SetFormPos(FParent);
		end;
	end;
end;

function TINIFormSaver.GetSectionName(const Name:string):string;
begin
	Result:=Name;

  if Assigned(FOnGetSectionName) then FOnGetSectionName(Self,Result);
end;

procedure TINIFormSaver.Notification(AComponent:TComponent;Operation:TOperation);
begin
	inherited Notification(AComponent,Operation);

  if (Operation=opRemove) and (AComponent=FSection) then FSection:=nil;
end;

procedure Register;
begin
	RegisterComponents('INIDb',[TINIFormSaver]);
end;

procedure TINIFormSaver.SetFormPos(Form:TForm);
var W,H,X,Y:Integer;
begin
  W:=Form.Width;
  H:=Form.Height;

  Form.WindowState:=TWindowState(FSection.FieldByName('State').AsInteger);

  X:=Min(FSection.FieldByName('X').AsInteger,Screen.Width-1);
  Y:=Min(FSection.FieldByName('Y').AsInteger,Screen.Height-1);

  if not (Form.BorderStyle in [bsSingle,bsDialog]) then begin
    W:=FSection.FieldByName('Width').AsInteger;
    H:=FSection.FieldByName('Height').AsInteger;
  end;

  if Form.WindowState<>wsMaximized then begin
    if (W<>Form.Width) or (H<>Form.Height) or (X<>Form.Left) or (Y<>Form.Top) then begin
      Form.SetBounds(X,Y,W,H);
    end;
  end;
end;

end.
